/* upstart
 *
 * com.ubuntu.Upstart.Job.c - Auto-generated d-bus bindings
 *
 * Copyright © 2008 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif /* HAVE_CONFIG_H */


#include <dbus/dbus.h>

#include <errno.h>
#include <limits.h>

#include <nih/macros.h>
#include <nih/alloc.h>
#include <nih/string.h>
#include <nih/logging.h>
#include <nih/error.h>
#include <nih/errors.h>
#include <nih/dbus.h>

#include "com.ubuntu.Upstart.Job.h"


int
job_class_get_instance (NihDBusProxy  *proxy,
                        char * const  *env,
                        char         **instance)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	DBusMessageIter  env_iter;
	size_t           env_len;
	char            *instance_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.ubuntu.Upstart.Job", "GetInstance");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	if (! dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY, "s", &env_iter)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	env_len = 0;
	for (char * const *env_p = env; env_p && *env_p; env_p++) {
		char *env_elem;
		char *env_elem_value;

		env_elem = *env_p;

		env_elem_value = env_elem;
		if (! dbus_message_iter_append_basic (&env_iter, DBUS_TYPE_STRING, &env_elem_value)) {
			dbus_message_unref (message);
			nih_return_no_memory_error (-1);

		}

		(env_len)++;
	}

	if (! dbus_message_iter_close_container (&iter, &env_iter)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_OBJECT_PATH) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &instance_value);

	*instance = nih_strdup (proxy, instance_value);
	if (! *instance) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
job_class_get_instance_by_name (NihDBusProxy  *proxy,
                                const char    *name,
                                char         **instance)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	const char      *name_value;
	char            *instance_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.ubuntu.Upstart.Job", "GetInstanceByName");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	name_value = name;
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &name_value)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_OBJECT_PATH) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &instance_value);

	*instance = nih_strdup (proxy, instance_value);
	if (! *instance) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
job_class_get_all_instances (NihDBusProxy   *proxy,
                             char         ***instances)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	DBusMessageIter  instances_iter;
	size_t           instances_len;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.ubuntu.Upstart.Job", "GetAllInstances");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);


	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_ARRAY) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	if (dbus_message_iter_get_element_type (&iter) != DBUS_TYPE_OBJECT_PATH) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_recurse (&iter, &instances_iter);

	*instances = NULL;
	instances_len = 0;

	while (dbus_message_iter_get_arg_type (&instances_iter) != DBUS_TYPE_INVALID) {
		char *instances_elem;
		char *instances_elem_value;

		if (dbus_message_iter_get_arg_type (&instances_iter) != DBUS_TYPE_OBJECT_PATH) {
			dbus_message_unref (reply);
			nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

		}

		dbus_message_iter_get_basic (&instances_iter, &instances_elem_value);

		instances_elem = nih_strdup (proxy, instances_elem_value);
		if (! instances_elem) {
			dbus_message_unref (reply);
			nih_return_no_memory_error (-1);

		}

		dbus_message_iter_next (&instances_iter);

		*instances = nih_realloc (*instances, proxy, sizeof (char *) * ((instances_len) + 1));
		if (! *instances) {
			dbus_message_unref (reply);
			nih_return_no_memory_error (-1);

		}

		(*instances)[(instances_len)++] = instances_elem;

	}

	dbus_message_iter_next (&iter);

	*instances = nih_realloc (*instances, proxy, sizeof (char *) * ((instances_len) + 1));
	if (! *instances) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	(*instances)[(instances_len)] = NULL;

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
job_class_start (NihDBusProxy  *proxy,
                 char * const  *env,
                 char         **instance)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	DBusMessageIter  env_iter;
	size_t           env_len;
	char            *instance_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.ubuntu.Upstart.Job", "Start");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	if (! dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY, "s", &env_iter)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	env_len = 0;
	for (char * const *env_p = env; env_p && *env_p; env_p++) {
		char *env_elem;
		char *env_elem_value;

		env_elem = *env_p;

		env_elem_value = env_elem;
		if (! dbus_message_iter_append_basic (&env_iter, DBUS_TYPE_STRING, &env_elem_value)) {
			dbus_message_unref (message);
			nih_return_no_memory_error (-1);

		}

		(env_len)++;
	}

	if (! dbus_message_iter_close_container (&iter, &env_iter)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_OBJECT_PATH) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &instance_value);

	*instance = nih_strdup (proxy, instance_value);
	if (! *instance) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
job_class_stop (NihDBusProxy *proxy,
                char * const *env)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	DBusMessageIter  env_iter;
	size_t           env_len;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.ubuntu.Upstart.Job", "Stop");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	if (! dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY, "s", &env_iter)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	env_len = 0;
	for (char * const *env_p = env; env_p && *env_p; env_p++) {
		char *env_elem;
		char *env_elem_value;

		env_elem = *env_p;

		env_elem_value = env_elem;
		if (! dbus_message_iter_append_basic (&env_iter, DBUS_TYPE_STRING, &env_elem_value)) {
			dbus_message_unref (message);
			nih_return_no_memory_error (-1);

		}

		(env_len)++;
	}

	if (! dbus_message_iter_close_container (&iter, &env_iter)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
job_class_restart (NihDBusProxy  *proxy,
                   char * const  *env,
                   char         **instance)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	DBusMessageIter  env_iter;
	size_t           env_len;
	char            *instance_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.ubuntu.Upstart.Job", "Restart");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	if (! dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY, "s", &env_iter)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	env_len = 0;
	for (char * const *env_p = env; env_p && *env_p; env_p++) {
		char *env_elem;
		char *env_elem_value;

		env_elem = *env_p;

		env_elem_value = env_elem;
		if (! dbus_message_iter_append_basic (&env_iter, DBUS_TYPE_STRING, &env_elem_value)) {
			dbus_message_unref (message);
			nih_return_no_memory_error (-1);

		}

		(env_len)++;
	}

	if (! dbus_message_iter_close_container (&iter, &env_iter)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_OBJECT_PATH) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &instance_value);

	*instance = nih_strdup (proxy, instance_value);
	if (! *instance) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

