/* upstart
 *
 * com.ubuntu.Upstart.c - Auto-generated d-bus bindings
 *
 * Copyright © 2008 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif /* HAVE_CONFIG_H */


#include <dbus/dbus.h>

#include <errno.h>
#include <limits.h>

#include <nih/macros.h>
#include <nih/alloc.h>
#include <nih/string.h>
#include <nih/logging.h>
#include <nih/error.h>
#include <nih/errors.h>
#include <nih/dbus.h>

#include "com.ubuntu.Upstart.h"


int
control_reload_configuration (NihDBusProxy *proxy)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.ubuntu.Upstart", "ReloadConfiguration");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);


	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
control_get_job_by_name (NihDBusProxy  *proxy,
                         const char    *name,
                         char         **job)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	const char      *name_value;
	char            *job_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.ubuntu.Upstart", "GetJobByName");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	name_value = name;
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &name_value)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_OBJECT_PATH) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &job_value);

	*job = nih_strdup (proxy, job_value);
	if (! *job) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
control_get_all_jobs (NihDBusProxy   *proxy,
                      char         ***jobs)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	DBusMessageIter  jobs_iter;
	size_t           jobs_len;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.ubuntu.Upstart", "GetAllJobs");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);


	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_ARRAY) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	if (dbus_message_iter_get_element_type (&iter) != DBUS_TYPE_OBJECT_PATH) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_recurse (&iter, &jobs_iter);

	*jobs = NULL;
	jobs_len = 0;

	while (dbus_message_iter_get_arg_type (&jobs_iter) != DBUS_TYPE_INVALID) {
		char *jobs_elem;
		char *jobs_elem_value;

		if (dbus_message_iter_get_arg_type (&jobs_iter) != DBUS_TYPE_OBJECT_PATH) {
			dbus_message_unref (reply);
			nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

		}

		dbus_message_iter_get_basic (&jobs_iter, &jobs_elem_value);

		jobs_elem = nih_strdup (proxy, jobs_elem_value);
		if (! jobs_elem) {
			dbus_message_unref (reply);
			nih_return_no_memory_error (-1);

		}

		dbus_message_iter_next (&jobs_iter);

		*jobs = nih_realloc (*jobs, proxy, sizeof (char *) * ((jobs_len) + 1));
		if (! *jobs) {
			dbus_message_unref (reply);
			nih_return_no_memory_error (-1);

		}

		(*jobs)[(jobs_len)++] = jobs_elem;

	}

	dbus_message_iter_next (&iter);

	*jobs = nih_realloc (*jobs, proxy, sizeof (char *) * ((jobs_len) + 1));
	if (! *jobs) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	(*jobs)[(jobs_len)] = NULL;

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
control_emit_event (NihDBusProxy *proxy,
                    const char   *name,
                    char * const *env)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	const char      *name_value;
	DBusMessageIter  env_iter;
	size_t           env_len;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.ubuntu.Upstart", "EmitEvent");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	name_value = name;
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &name_value)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	if (! dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY, "s", &env_iter)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	env_len = 0;
	for (char * const *env_p = env; env_p && *env_p; env_p++) {
		char *env_elem;
		char *env_elem_value;

		env_elem = *env_p;

		env_elem_value = env_elem;
		if (! dbus_message_iter_append_basic (&env_iter, DBUS_TYPE_STRING, &env_elem_value)) {
			dbus_message_unref (message);
			nih_return_no_memory_error (-1);

		}

		(env_len)++;
	}

	if (! dbus_message_iter_close_container (&iter, &env_iter)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
control_get_version (NihDBusProxy  *proxy,
                     char         **version)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	char            *version_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.ubuntu.Upstart", "GetVersion");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);


	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRING) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &version_value);

	*version = nih_strdup (proxy, version_value);
	if (! *version) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
control_set_log_prio (NihDBusProxy *proxy,
                      int32_t       priority)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.ubuntu.Upstart", "SetLogPrio");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_INT32, &priority)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

